{{
*************************************
* EEPROM upper 32K boot module
*************************************
}}

CON
   _clkmode  = xtal1 + pll16x
   _clkfreq  = 80_000_000

  BS = 8
  CR = $A
  COGMAX = 8
  INTERPRETER_ADDRESS = $f004

  BOOTADDR = $8000
  
OBJ
  ee  : "PasmEEPROM"
  fds : "FullDuplexSingleton"
  boo : "eepromPasmLoader"
  
CON
  PROGLEN = 25000
  
VAR
  byte heap[PROGLEN]
  byte proghdr[16]
  
  long eeSetup, eeClkLow, eeClkHigh
  
PUB main | size

  fds.start(31, 30, 0, 115200)
  ee.start(28, 29)
  waitcnt(clkfreq+cnt)

  print(string(CR,CR,"Bootloader started.",CR))
  waitcnt(clkfreq+cnt)

  size := readHeader(BOOTADDR)

  dumpSize(BOOTADDR, size)
  
  eeSetup   := ((_clkfreq / 10000) * 250) / 100000
  eeClkLow  := ((_clkfreq / 10000) * 1500) / 100000
  eeClkHigh := ((_clkfreq / 10000) * 1500) / 100000

  print(string(CR,CR, "eeSetup:eeClkLow:eeClkHigh "))
  dec(eeSetup)
  putchar(" ")
  dec(eeClkLow)
  putchar(" ")
  dec(eeClkHigh)
  putchar(CR)
  
  boo.start(BOOTADDR, size, eeSetup, eeClkLow, eeClkHigh)
  waitcnt(clkfreq+cnt)
  
  print(string(CR, "After boo.start() CLKFREQ "))
  dec(clkfreq)
  print(string(" Size "))
  dec(size)
  print(string(CR, "StackMarkers "))
  hex(long[size-8], 8)
  putchar(" ")
  hex(long[size-4], 8)
  putchar(CR)
  
  repeat
  
  {
  repeat
   ' boot from eeprom
    rc := \boot(BOOTADDR)
    ifnot rc
      repeat
    else
      print(string(CR,"Boot failed "))
      dec(rc)
  }

VAR long cogTestStack[4]

PRI cogTest
  return 0

PRI getFreeCog
  result := cognew(cogtest,@cogTestStack)
  waitcnt(clkfreq/10+cnt)
  cogstop(result)

PRI boot(addr) | ii, sbuf[100], rc, len, sdpin, spinbin

  spinbin := loadProgram(addr)
  return runProg(spinbin)

      
VAR long pcog         ' program cog variable
    
PRI runProg(spinptr) | ii, cogs

  if spinptr
   
      if pcog > 0
        cogstop(pcog~ - 1)

      cogs := COGMAX-getFreeCog        

      newline   ' last chance for output

      ' kill all cogs except main interpreter
      repeat while cogs > 0
        cogstop(cogs--)

      pcog := cognew(INTERPRETER_ADDRESS, spinptr+4) + 1
      waitcnt(clkfreq/10+cnt)

PRI loadError(str,maddr)
  print(str)
  hex(maddr, 4)

PRI readHeader(maddr) | ii
  repeat ii from 0 to 15
    byte[@proghdr][ii] := ee.readByte(maddr+ii)
  result := word[@proghdr+$A] ' plen at stack base gives enough room for prog

PRI dumpSize(maddr, plen) 
  print(string(CR,"EE ADDR "))
  hex(maddr, 4)
  print(string(" size "))
  dec(plen)
  newline
         
PRI loadProgram(maddr) | rc,ii,plen,progptr
  plen := readHeader(maddr)
   
  dumpSize(maddr, plen)
  
 
  if plen > PROGLEN
    return loadError(string(CR,"Program too big"), plen)
        
  progptr := @heap
  progptr := (progptr + 4) & !3
  
  repeat ii from 0 to plen
    byte[progptr+ii] := ee.readByte(maddr+ii)
  
  ' Fixup
  Repeat ii from 0 to 4
    Word[progptr+6+ii<<1] += progptr

  ' Clear varspace
  rc := word[@proghdr+$8]
  Repeat ii from rc to plen step 4
    long[progptr+ii] := 0
    
  long[progptr+plen-4] := $fff9ffff
  long[progptr+plen-8] := $fff9ffff

  return progptr


PRI getChar | id
  return fds.rx

PRI iskey
  return fds.rxcheck+1

PRI newline
    putchar(CR)
  
PRI putchar(ch)
  fds.tx(ch)

PRI puts(str)
  fds.str(str)
  newline

PRI print(str)
  fds.str(str)

PRI dec(v)
  fds.dec(v)
    
PRI hex(v,len)
  fds.hex(v,len)

